@@ -55,7 +55,7 @@ gdk_pixmap_new (GdkWindow *window,
     return NULL;
 
   if (depth == -1)
-    gdk_window_get_geometry (window, NULL, NULL, NULL, NULL, &depth);
+    depth = gdk_window_get_visual (window)->depth;
 
   private = g_new (GdkWindowPrivate, 1);
   pixmap = (GdkPixmap*) private;
@@ -64,6 +64,7 @@ gdk_pixmap_new (GdkWindow *window,
   private->window_type = GDK_WINDOW_PIXMAP;
   private->xwindow = XCreatePixmap (private->xdisplay, window_private->xwindow,
 				    width, height, depth);
+  private->colormap = NULL;
   private->parent = NULL;
   private->x = 0;
   private->y = 0;
@@ -148,7 +149,7 @@ gdk_pixmap_create_from_data (GdkWindow *window,
     return NULL;
 
   if (depth == -1)
-    gdk_window_get_geometry (window, NULL, NULL, NULL, NULL, &depth);
+    depth = gdk_window_get_visual (window)->depth;
 
   private = g_new (GdkWindowPrivate, 1);
   pixmap = (GdkPixmap*) private;
@@ -255,8 +256,14 @@ gdk_pixmap_read_string (FILE  *infile,
     {
       if (cnt == (*buffer_size))
 	{
-	  (*buffer_size) *= 2;
- 	  (*buffer) = (gchar *) g_realloc ((*buffer), *buffer_size);	}
+	  guint new_size = (*buffer_size) * 2;
+	  if (new_size > (*buffer_size))
+	    *buffer_size = new_size;
+	  else
+	    return FALSE;
+	  
+ 	  (*buffer) = (gchar *) g_realloc ((*buffer), *buffer_size);	
+	}
 
       if (c != '"')
         (*buffer)[cnt++] = c;
@@ -398,6 +405,11 @@ gdk_pixmap_colormap_create_from_xpm (GdkWindow   *window,
               gdk_pixmap_read_string (infile, &buffer, &buffer_size);
 
               sscanf (buffer,"%d %d %d %d", &width, &height, &num_cols, &cpp);
+	      if (cpp >= 32)
+		{
+		  g_warning ("Pixmap has more than 31 characters per color\n");
+		  return NULL;
+		}
 
               colors = g_new(_GdkPixmapColor, num_cols);
 
@@ -573,6 +585,11 @@ gdk_pixmap_colormap_create_from_xpm_d (GdkWindow  *window,
   i = 0;
   buffer = data[i++];
   sscanf (buffer,"%d %d %d %d", &width, &height, &num_cols, &cpp);
+  if (cpp >= 32)
+    {
+      g_warning ("Pixmap has more than 31 characters per color\n");
+      return NULL;
+    }
 
   colors = g_new(_GdkPixmapColor, num_cols);
 
