@@ -110,10 +110,15 @@ static guint container_signals[LAST_SIGNAL] = { 0 };
 
 static GtkWidgetClass *parent_class = NULL;
 
-guint
-gtk_container_get_type ()
+static const gchar *vadjustment_key = "gtk-vadjustment";
+static guint        vadjustment_key_id = 0;
+static const gchar *hadjustment_key = "gtk-hadjustment";
+static guint        hadjustment_key_id = 0;
+
+GtkType
+gtk_container_get_type (void)
 {
-  static guint container_type = 0;
+  static GtkType container_type = 0;
 
   if (!container_type)
     {
@@ -142,7 +147,11 @@ gtk_container_class_init (GtkContainerClass *class)
 
   object_class = (GtkObjectClass*) class;
   widget_class = (GtkWidgetClass*) class;
+
   parent_class = gtk_type_class (gtk_widget_get_type ());
+
+  vadjustment_key_id = gtk_object_data_force_id (vadjustment_key);
+  hadjustment_key_id = gtk_object_data_force_id (hadjustment_key);
   
   gtk_object_add_arg_type ("GtkContainer::border_width", GTK_TYPE_LONG, GTK_ARG_READWRITE, ARG_BORDER_WIDTH);
   gtk_object_add_arg_type ("GtkContainer::auto_resize", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_AUTO_RESIZE);
@@ -350,9 +359,6 @@ gtk_container_remove (GtkContainer *container,
   g_return_if_fail (widget->parent == GTK_WIDGET (container));
   
   gtk_signal_emit (GTK_OBJECT (container), container_signals[REMOVE], widget);
-
-  if (container->focus_child == widget)
-    container->focus_child = NULL;
 }
 
 void
@@ -529,6 +535,47 @@ gtk_container_unregister_toplevel (GtkContainer *container)
   gtk_widget_unref (GTK_WIDGET (container));
 }
 
+void
+gtk_container_set_focus_child (GtkContainer     *container,
+			       GtkWidget        *child)
+{
+  g_return_if_fail (container != NULL);
+  g_return_if_fail (GTK_IS_CONTAINER (container));
+  if (child)
+    g_return_if_fail (GTK_IS_WIDGET (child));
+
+  if (child != container->focus_child)
+    {
+      if (container->focus_child)
+	gtk_widget_unref (container->focus_child);
+      container->focus_child = child;
+      if (container->focus_child)
+	gtk_widget_ref (container->focus_child);
+    }
+
+
+  /* check for h/v adjustments
+   */
+  if (container->focus_child)
+    {
+      GtkAdjustment *adjustment;
+      
+      adjustment = gtk_object_get_data_by_id (GTK_OBJECT (container), vadjustment_key_id);
+      if (adjustment)
+	gtk_adjustment_clamp_page (adjustment,
+				   container->focus_child->allocation.y,
+				   (container->focus_child->allocation.y +
+				    container->focus_child->allocation.height));
+
+      adjustment = gtk_object_get_data_by_id (GTK_OBJECT (container), hadjustment_key_id);
+      if (adjustment)
+	gtk_adjustment_clamp_page (adjustment,
+				   container->focus_child->allocation.x,
+				   (container->focus_child->allocation.x +
+				    container->focus_child->allocation.width));
+    }
+}
+
 static void
 gtk_container_marshal_signal_1 (GtkObject      *object,
 				GtkSignalFunc   func,
@@ -972,7 +1019,7 @@ gtk_container_focus_move (GtkContainer     *container,
   GtkWidget *child;
 
   focus_child = container->focus_child;
-  container->focus_child = NULL;
+  gtk_container_set_focus_child (container, NULL);
 
   while (children)
     {
@@ -1059,3 +1106,38 @@ gtk_container_hide_all (GtkWidget *widget)
   gtk_container_foreach (container, (GtkCallback) gtk_widget_hide_all, NULL);
 }
 
+void
+gtk_container_set_focus_vadjustment (GtkContainer  *container,
+				     GtkAdjustment *adjustment)
+{
+  g_return_if_fail (container != NULL);
+  g_return_if_fail (GTK_IS_CONTAINER (container));
+  if (adjustment)
+    g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+
+  if (adjustment)
+    gtk_object_ref (adjustment);
+
+  gtk_object_set_data_by_id_full (GTK_OBJECT (container),
+				  vadjustment_key_id,
+				  adjustment,
+				  (GtkDestroyNotify) gtk_object_unref);
+}
+
+void
+gtk_container_set_focus_hadjustment (GtkContainer  *container,
+				     GtkAdjustment *adjustment)
+{
+  g_return_if_fail (container != NULL);
+  g_return_if_fail (GTK_IS_CONTAINER (container));
+  if (adjustment)
+    g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+
+  if (adjustment)
+    gtk_object_ref (adjustment);
+
+  gtk_object_set_data_by_id_full (GTK_OBJECT (container),
+				  hadjustment_key_id,
+				  adjustment,
+				  (GtkDestroyNotify) gtk_object_unref);
+}
