@@ -12,8 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 #include <ctype.h>
 #include <string.h>
@@ -440,7 +441,7 @@ gtk_editable_selection_received  (GtkWidget         *widget,
     {
     case STRING:
       selection_data->data[selection_data->length] = 0;
-      gtk_editable_insert_text (editable, selection_data->data,
+      gtk_editable_insert_text (editable, (gchar *)selection_data->data,
 				strlen ((gchar *)selection_data->data), 
 				&tmp_pos);
       editable->current_pos = tmp_pos;
