@@ -268,7 +268,7 @@ gtk_label_size_request (GtkWidget      *widget,
   	  width = MAX (width,
                        gdk_text_width (GTK_WIDGET (label)->style->font,
 				       row->data,
-                                       (gchar*) row->next->data - (gchar*) row->data) - 1);
+                                       (gchar*) row->next->data - (gchar*) row->data - 1));
       else
         width = MAX (width, gdk_string_width (GTK_WIDGET (label)->style->font, row->data));
       row = row->next;
@@ -396,13 +396,15 @@ static void
 gtk_label_state_changed (GtkWidget      *widget,
 			 guint	         previous_state)
 {
-  GTK_LABEL (widget)->needs_clear = 1;
+  if (GTK_WIDGET_DRAWABLE (widget))
+    GTK_LABEL (widget)->needs_clear = 1;
 }
 
 static void 
 gtk_label_style_set	(GtkWidget      *widget,
 			 GtkStyle       *previous_style)
 {
-  GTK_LABEL (widget)->needs_clear = 1;
+  if (GTK_WIDGET_DRAWABLE (widget))
+    GTK_LABEL (widget)->needs_clear = 1;
 }
 
