@@ -12,8 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 #include "gtkcontainer.h"
 #include "gtkmisc.h"
@@ -24,7 +25,7 @@ enum {
   ARG_XALIGN,
   ARG_YALIGN,
   ARG_XPAD,
-  ARG_YPAD,
+  ARG_YPAD
 };
 
 static void gtk_misc_class_init (GtkMiscClass *klass);
