@@ -12,8 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 #ifndef __GTK_TYPE_UTILS_H__
 #define __GTK_TYPE_UTILS_H__
@@ -83,7 +84,7 @@ typedef void (*GtkArgSetFunc) (GtkObject *object, GtkArg *arg, guint arg_id);
 typedef gint (*GtkFunction) (gpointer data);
 typedef void (*GtkCallbackMarshal) (GtkObject *object,
 				    gpointer data,
-				    int n_args,
+				    guint n_args,
 				    GtkArg *args);
 typedef void (*GtkDestroyNotify) (gpointer data);
 
