@@ -12,8 +12,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 #include "gtksignal.h"
 #include "gtkviewport.h"
@@ -480,7 +481,7 @@ gtk_viewport_add (GtkContainer *container,
     {
       gtk_widget_set_parent (widget, GTK_WIDGET (container));
       gtk_widget_set_parent_window (widget, GTK_VIEWPORT (container)->bin_window);
-      if (GTK_WIDGET_VISIBLE (widget->parent))
+      if (GTK_WIDGET_VISIBLE (widget))
 	{
 	  if (GTK_WIDGET_MAPPED (widget->parent) &&
 	      !GTK_WIDGET_MAPPED (widget))
